@@ -15,6 +15,7 @@ use Locale::Maketext::Simple    Style => 'gettext';
 
 ### solaris has silly /bin/tar output ###
 use constant ON_SOLARIS     => $^O eq 'solaris' ? 1 : 0;
+use constant ON_NETBSD      => $^O eq 'netbsd' ? 1 : 0;
 use constant FILE_EXISTS    => sub { -e $_[0] ? 1 : 0 };
 
 ### VMS may require quoting upper case command options
@@ -43,7 +44,7 @@ use vars qw[$VERSION $PREFER_BIN $PROGRAMS $WARN $DEBUG
             $_ALLOW_BIN $_ALLOW_PURE_PERL $_ALLOW_TAR_ITER
          ];
 
-$VERSION            = '0.44';
+$VERSION            = '0.48';
 $PREFER_BIN         = 0;
 $WARN               = 1;
 $DEBUG              = 0;
@@ -125,6 +126,12 @@ See the C<HOW IT WORKS> section further down for details.
 ### see what /bin/programs are available ###
 $PROGRAMS = {};
 for my $pgm (qw[tar unzip gzip bunzip2 uncompress unlzma unxz]) {
+    if ( $pgm eq 'unzip' and ON_NETBSD ) {
+      local $IPC::Cmd::INSTANCES = 1;
+      my @possibles = can_run($pgm);
+      ($PROGRAMS->{$pgm}) = grep { m!/usr/pkg/! } can_run($pgm);
+      next;
+    }
     $PROGRAMS->{$pgm} = can_run($pgm);
 }
 
@@ -142,7 +149,7 @@ my $Mapping = {  # binary program           # pure perl module
     is_txz  => { bin => '_untar_bin',       pp => '_untar_at'   },
 };
 
-{   ### use subs so we re-generate array refs etc for the no-overide flags
+{   ### use subs so we re-generate array refs etc for the no-override flags
     ### if we don't, then we reuse the same arrayref, meaning objects store
     ### previous errors
     my $tmpl = {
@@ -152,7 +159,7 @@ my $Mapping = {  # binary program           # pure perl module
         _error_msg_long => sub { { no_override => 1, default => [] }        },
     };
 
-    ### build accesssors ###
+    ### build accessors ###
     for my $method( keys %$tmpl, 
                     qw[_extractor _gunzip_to files extract_path],
     ) {
@@ -209,7 +216,7 @@ Corresponds to a C<.bz2> suffix.
 
 =item tbz
 
-Bzip2 compressed tar file, as produced by, for exmample C</bin/tar -j>.
+Bzip2 compressed tar file, as produced by, for example C</bin/tar -j>.
 Corresponds to a C<.tbz> or C<.tar.bz2> suffix.
 
 =item lzma
@@ -224,7 +231,7 @@ Corresponds to a C<.xz> suffix.
 
 =item txz
 
-Xz compressed tar file, as produced by, for exmample C</bin/tar -J>.
+Xz compressed tar file, as produced by, for example C</bin/tar -J>.
 Corresponds to a C<.txz> or C<.tar.xz> suffix.
 
 =back
@@ -709,7 +716,7 @@ sub have_old_bunzip2 {
                                                 \s+ [\d,.]+ \s tape \s blocks
                                             |x ? $1 : $_);
     
-                        ### only STDOUT, see above. Sometims, extra whitespace
+                        ### only STDOUT, see above. Sometimes, extra whitespace
                         ### is present, so make sure we only pick lines with
                         ### a length
                         } grep { length } map { split $/, $_ } @{$out[3]};     
@@ -1060,7 +1067,7 @@ sub _unzip_bin {
 
     ### first, get the files.. it must be 2 different commands with 'unzip' :(
     {   ### on VMS, capital letter options have to be quoted. This is
-        ### peported by John Malmberg on P5P Tue 21 Aug 2007 05:05:11 
+        ### reported by John Malmberg on P5P Tue 21 Aug 2007 05:05:11
         ### Subject: [patch@31735]Archive Extract fix on VMS.
         my $opt = ON_VMS ? '"-Z"' : '-Z';
         my $cmd = [ $self->bin_unzip, $opt, '-1', $self->archive ];
@@ -1144,7 +1151,7 @@ sub _unzip_az {
     for my $member ($zip->members) {
         push @files, $member->{fileName};
 
-        ### file to extact to, to avoid the above problem
+        ### file to extract to, to avoid the above problem
         my $to = File::Spec->catfile( $extract_dir, $member->{fileName} );
         
         unless( $zip->extractMember($member, $to) == &Archive::Zip::AZ_OK ) {
