@@ -8,6 +8,9 @@
 # from information hardcoded into this script (the $tree hash), plus the
 # template for warnings.pm in the DATA section.
 #
+# When changing the number of warnings, t/op/caller.t should change to
+# correspond with the value of $BYTES in lib/warnings.pm
+#
 # With an argument of 'tree', just dump the contents of $tree and exits.
 # Also accepts the standard regen_lib -q and -v args.
 #
@@ -71,7 +74,11 @@ my $tree = {
 	'closure'	=> [ 5.008, DEFAULT_OFF],
 	'overflow'	=> [ 5.008, DEFAULT_OFF],
 	'portable'	=> [ 5.008, DEFAULT_OFF],
-	'utf8'		=> [ 5.008, DEFAULT_OFF],
+	'utf8'		=> [ 5.008, {
+                                'surrogate' => [ 5.013, DEFAULT_OFF],
+                                'nonchar' => [ 5.013, DEFAULT_OFF],
+                                'non_unicode' => [ 5.013, DEFAULT_OFF],
+                        }],
        	'exiting'	=> [ 5.008, DEFAULT_OFF],
        	'pack'		=> [ 5.008, DEFAULT_OFF],
        	'unpack'	=> [ 5.008, DEFAULT_OFF],
@@ -82,15 +89,6 @@ my $tree = {
   	}],
 } ;
 
-###########################################################################
-sub tab {
-    my($l, $t) = @_;
-    $t .= "\t" x ($l - (length($t) + 1) / 8);
-    $t;
-}
-
-###########################################################################
-
 my %list ;
 my %Value ;
 my %ValueToName ;
@@ -262,16 +260,11 @@ if (@ARGV && $ARGV[0] eq "tree")
     exit ;
 }
 
-my $warn = safer_open("warnings.h-new");
-my $pm = safer_open("lib/warnings.pm-new");
-
-print $warn <<'EOM' ;
-/* -*- buffer-read-only: t -*-
-   !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
-   This file is built by regen/warnings.pl
-   Any changes made here will be lost!
-*/
+my $warn = safer_open('warnings.h-new', 'warnings.h');
+my $pm = safer_open('lib/warnings.pm-new', 'lib/warnings.pm');
 
+print $pm read_only_top(lang => 'Perl', by => 'regen/warnings.pl');
+print $warn read_only_top(lang => 'C', by => 'regen/warnings.pl'), <<'EOM';
 
 #define Off(x)			((x) / 8)
 #define Bit(x)			(1 << ((x) % 8))
@@ -371,11 +364,9 @@ print $warn <<'EOM';
 	      isWARNf_on(PL_curcop->cop_warnings, unpackWARN4(x))))
 
 /* end of file warnings.h */
-/* ex: set ro: */
 EOM
 
-safer_close $warn;
-rename_if_different("warnings.h-new", "warnings.h");
+read_only_bottom_close_and_rename($warn);
 
 while (<DATA>) {
     last if /^KEYWORDS$/ ;
@@ -431,20 +422,12 @@ while (<DATA>) {
     print $pm $_ ;
 }
 
-print $pm "# ex: set ro:\n";
-safer_close $pm;
-rename_if_different("lib/warnings.pm-new", "lib/warnings.pm");
+read_only_bottom_close_and_rename($pm);
 
 __END__
-# -*- buffer-read-only: t -*-
-# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
-# This file was created by regen/warnings.pl
-# Any changes made here will be lost.
-#
-
 package warnings;
 
-our $VERSION = '1.11';
+our $VERSION = '1.12';
 
 # Verify that we're called correctly so that warnings will work.
 # see also strict.pm.
